/*
 * lcdcontrol.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>

static const char *VERSION        = "0.0.2";
static const char *DESCRIPTION    = "A plugin to control your TV via serial port";
static const char *MAINMENUENTRY  = "LCD Control";

#define MANUALSTART       600
static const char *SerialPorts[] = { "/dev/ttyS0", "/dev/ttyS1"};

// ------------------------------------- Serial Port Handling---------------------------------------------------

class cSerialPort {
private:
  int fd;
  struct termios options;
public:
  void openPort(char *serialPort);
  void closePort();
  void writePort(char *cmd);  
};

void cSerialPort::openPort(char *serialPort){
	
        fd = open(serialPort, O_RDWR | O_NOCTTY | O_NDELAY);
        fcntl(fd, F_SETFL, 0);

        tcgetattr(fd, &options);

        options.c_cflag &= ~PARENB;
        options.c_cflag &= ~CSTOPB;
        options.c_cflag &= ~CSIZE;
        options.c_cflag |= CS8;
        cfsetispeed(&options, B9600);
        cfsetospeed(&options, B9600);

        tcsetattr(fd, TCSANOW, &options);
}

void cSerialPort::writePort(char *cmd){
	int laenge;
	laenge = strlen(cmd);
	write(fd, cmd, laenge);
}

void cSerialPort::closePort(){
	close(fd);
}

// ------------------------------------- Serial Port Handling---------------------------------------------------

// ------------------------------------- LcdControlSetup -----------------------------------------------------------
class cLcdControlSetup {
	public:
	int serialPort;
	int autoOn;
    cLcdControlSetup();
    bool SetupParse(const char *Name, const char *Value);
};

cLcdControlSetup::cLcdControlSetup(void)
{
  serialPort = 1;
  autoOn = 0;
}

bool cLcdControlSetup::SetupParse(const char *Name, const char *Value){
	if (!strcasecmp(Name, "serialPort"))      serialPort = atoi(Value);
	else if (!strcasecmp(Name, "autoOn"))     autoOn = atoi(Value);
    else
      return false;
    return true;
}
// ------------------------------------- LcdControlSetup -----------------------------------------------------------

// ------------------------------------- MenuLcdControl -----------------------------------------------------------

class cMenuLcdControl : public cOsdMenu {
private:
	cSerialPort port;
public:
	cMenuLcdControl();
	virtual eOSState ProcessKey(eKeys Key);
};

cMenuLcdControl::cMenuLcdControl():cOsdMenu("Lcdcontrol") {
	Add(new cOsdItem("16:9"));
 	Add(new cOsdItem("Auto"));
}

eOSState cMenuLcdControl::ProcessKey(eKeys Key) {
	
	eOSState state = cOsdMenu::ProcessKey(Key);

	if (HasSubMenu()) return osContinue;
    if (Key == kOk){
    	switch (cOsdMenu::Current ()) {
    		case 0: {
      			dsyslog("Switching TV to 16:9");
      			port.openPort("/dev/ttyS1");
      			port.writePort("kc 01 2\n");
      			port.closePort();
      			return osEnd;
      			break;
    		}
    		case 1: {
      			dsyslog("Switching TV to Auto");
      			port.openPort("/dev/ttyS1");
      			port.writePort("kc 01 6\n");
      			port.closePort();
      			return osEnd;
      			break;
    		}
    		default: break;
		}
    }
	return state;
}
  
// ------------------------------------- MenuLcdControl -----------------------------------------------------------

// ------------------------------------- MenuLcdControlSetup ------------------------------------------------------

class cMenuLcdControlSetup : public cMenuSetupPage {
private:
  cLcdControlSetup config;
protected:
  virtual void Store(void);
public:
  cMenuLcdControlSetup(cLcdControlSetup c);
};


cMenuLcdControlSetup::cMenuLcdControlSetup(cLcdControlSetup c)
{
  config = c ;
  Add(new cMenuEditStraItem("Serial Port", &config.serialPort, 2 , SerialPorts));
  Add(new cMenuEditBoolItem("Switch TV on at Startup", &config.autoOn, "no", "yes"));    
}

void cMenuLcdControlSetup::Store(void)
{
    SetupStore("serialPort", config.serialPort);
    SetupStore("autoOn", config.autoOn);
}

// ------------------------------------- MenuLcdControlSetup ------------------------------------------------------

class cPluginLcdcontrol : public cPlugin {
private:
  cLcdControlSetup LcdControlSetup;
  cSerialPort port;
public:
  cPluginLcdcontrol(void);
  virtual ~cPluginLcdcontrol();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual cString Active(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Service(const char *Id, void *Data = NULL);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

cPluginLcdcontrol::cPluginLcdcontrol(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginLcdcontrol::~cPluginLcdcontrol()
{
  // Clean up after yourself!
}

const char *cPluginLcdcontrol::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginLcdcontrol::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginLcdcontrol::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginLcdcontrol::Start(void)
{
  // Start any background activities the plugin shall perform.
  time_t Now = time(NULL);
  cTimer *timer = Timers.GetNextActiveTimer();
  time_t Next  = timer ? timer->StartTime() : 0;
  time_t Delta = timer ? Next - Now : 0;
  if ( (!timer || Delta > MANUALSTART) && LcdControlSetup.autoOn == 1) {
  	port.openPort("/dev/ttyS1");
    port.writePort("ka 01 1\n");
    port.closePort();
  }
  return true;
}

void cPluginLcdcontrol::Stop(void)
{
  // Stop any background activities the plugin shall perform.
}

void cPluginLcdcontrol::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

void cPluginLcdcontrol::MainThreadHook(void)
{
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!
}

cString cPluginLcdcontrol::Active(void)
{
  // Return a message string if shutdown should be postponed
  return NULL;
}

cOsdObject *cPluginLcdcontrol::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  //return NULL;
  return new cMenuLcdControl();
}

cMenuSetupPage *cPluginLcdcontrol::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuLcdControlSetup(LcdControlSetup);
}

bool cPluginLcdcontrol::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return LcdControlSetup.SetupParse(Name, Value);
}

bool cPluginLcdcontrol::Service(const char *Id, void *Data)
{
  // Handle custom service requests from other plugins
  return false;
}

const char **cPluginLcdcontrol::SVDRPHelpPages(void)
{
  // Return help text for SVDRP commands this plugin implements
  return NULL;
}

cString cPluginLcdcontrol::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
  // Process SVDRP commands this plugin implements
  return NULL;
}

VDRPLUGINCREATOR(cPluginLcdcontrol); // Don't touch this!
